///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Collections.Generic;
using System.Collections; 
using System.Text;
using ILNumerics.Drawing; 
using ILNumerics.Drawing.Interfaces; 
using ILNumerics.Drawing.Shapes;
using ILNumerics; 

namespace ILNumerics.Drawing.Graphs {
    /// <summary>
    /// The class implements a scene graph (tree), capable 
    /// of holding, managing and drawing all kinds of shapes.
    /// </summary>
    public class ILSceneGraph : ILGraph {

        #region eventing
        /// <summary>
        /// fires, once somewhere in the graph a node was added
        /// </summary>
        public event SceneGraphNodeHandler NodeAdded; 
        void OnNodeAdded(ILSceneGraphNodeEventArgs args) {
            if (NodeAdded != null) {
                NodeAdded(this, args);  
            }
        }
        public event SceneGraphNodeHandler NodeRemoved; 
        void OnNodeRemoved(ILSceneGraphNodeEventArgs args) {
            if (NodeRemoved != null) {
                NodeRemoved(this, args);  
            }
        }

        void m_root_Invalidated(object sender, EventArgs e) {
            base.Invalidate();
        }
        void m_root_SizeChanged(object sender, EventArgs e) {
            m_localClipping.Set(m_root.PositionMin, m_root.PositionMax);
        }
        void m_root_NodeAdded(object sender, ILSceneGraphNodeEventArgs args) {
            OnNodeAdded(args); 
        }
        void m_root_NodeRemoved(object sender, ILSceneGraphNodeEventArgs args) {
            OnNodeRemoved(args);
        }

        #endregion

        #region attributes
        private ILSceneGraphRoot m_root; 
        #endregion

        #region properties
        /// <summary>
        /// root node of the scene graph
        /// </summary>
        public ILSceneGraphInnerNode Root {
            get {return m_root; }
        }
        #endregion

        #region constructors
        /// <summary>
        /// (internal) create new scene graph 
        /// </summary>
        /// <param name="panel">panel hosting the scene</param>
        /// <param name="clipping">clipping data object, usually member of the panel</param>
        internal ILSceneGraph (ILPanel panel, ILClippingData clipping) 
            : base(panel,clipping) {
            m_root = new ILSceneGraphRoot(panel);
            m_root.Invalidated += new EventHandler(m_root_Invalidated);
            m_root.SizeChanged += new EventHandler(m_root_SizeChanged);
            m_root.NodeAdded += new SceneGraphNodeHandler(m_root_NodeAdded);
            m_root.NodeRemoved += new SceneGraphNodeHandler(m_root_NodeRemoved);
            m_graphType = GraphType.SceneGraph; 
        }

        #endregion

        #region public interface
        /// <summary>
        /// Determines if this graph renders in 3D, may overridden in derived class
        /// </summary>
        /// <returns></returns>
        public override bool Is3DGraph() {
            return true;
        }
        /// <summary>
        /// (internal use) draws the whole scene graph
        /// </summary>
        /// <param name="p"></param>
        public override void Draw(ILRenderProperties p) {
            m_root.Draw(p); 
        }
        /// <summary>
        /// (internal use) configures and prepares the scene graph for rendering
        /// </summary>
        internal override void Configure() {
            if (!m_isReady) {
                m_root.Configure();
                m_isReady = true;
            }
        }
        /// <summary>
        /// add new node to root node
        /// </summary>
        /// <param name="node"></param>
        public void AddNode(ILSceneGraphNode node) {
            AddNode(node, m_root);
        }
        /// <summary>
        /// add a shape to the root node
        /// </summary>
        /// <param name="shape"></param>
        public void AddNode(ILShape shape) {
            ILSceneGraphShapedLeaf node = new ILSceneGraphShapedLeaf(m_panel);
            node.Shape = shape; 
            AddNode(node, m_root);
        }
        /// <summary>
        /// add new node as child of another node
        /// </summary>
        /// <param name="node"></param>
        /// <param name="parent"></param>
        public void AddNode(ILSceneGraphNode node, ILSceneGraphInnerNode parent) {
            parent.Add(node);             
        }
        /// <summary>
        /// collect all shapes contained in any scene graph nodes
        /// </summary>
        /// <returns></returns>
        public List<ILShape> GetAllShapes() {
            List<ILShape> ret = new List<ILShape>();
            getAllShapes(m_root, ret);
            return ret;  
        }
        #endregion

        #region private helpers
        private void getAllShapes(ILSceneGraphNode node, List<ILShape> ret) {
            foreach (ILSceneGraphNode n in node) {
                if (n is ILSceneGraphShapedLeaf) {
                    ILSceneGraphShapedLeaf snode = n as ILSceneGraphShapedLeaf;
                    if (snode.Shape != null)
                        ret.Add(snode.Shape);
                }
                //getAllShapes(n,ret); 
            }
        }

        #endregion

        #region IEnumerable<ILShape> Member

        //public IEnumerator<ILShape> GetEnumerator() {
        //    List<ILShape> ret = new List<ILShape>(); 
        //    getAllShapes(m_root,ret); 
        //    return ret.GetEnumerator();  
        //}

        //#endregion

        //#region IEnumerable Member

        //IEnumerator GetEnumerator() {
        //    List<ILShape> ret = new List<ILShape>(); 
        //    getAllShapes(m_root,ret); 
        //    return ret.GetEnumerator();  
        //}

        //#endregion

        //#region IILPanelConfigurator
        //public override void ConfigurePanel(ILPanel panel) {
        //    panel.InteractiveMode = InteractiveModes.Rotating;
        //    panel.DefaultView.Set(5.8f, 1.17f, panel.DefaultView.Distance);
        //}
        #endregion

    }
}
